describe('ContainerDecoratorPreference', function() {
	var zhPromise = new ZHPromise(Q);
	var dataSourceHolder = new DataSourceHolder(zhPromise, {}, {});
	var zhUtil = new ZhUtil({}, {}, {}, dataSourceHolder);
	var thenable = function(data) {
		return {
			then : function(fn){
				var dataReturned;
				if(fn){
					dataReturned = fn(data);
				}
				return thenable(dataReturned || data);
			}
		};
	};

	var screenService = {
		i18n: function(text) {
			return text;
		}
	};

	var localStorageEngine;
	var requestDataSourceEngine;
	var synchronizableStorageEngineFactory;
	var synchronizableStorageEngine;
	var preferenceManager;
	var containerRepository;
	var containerDecoratorPreference;

	var defaultContainer = {
		"name": "login",
		"showFooter": true,
		"showMenu": true,
		"widgets": [
			{
				"id": 1,
				"name": "loginWidget",
				"template": "widget/list.html",
				"actions": [
					{
						"id": 2,
						"name": "TEST",
						"showAsAction": "always",
						"events": [
							{
								"id": 3,
								"name": "ActionEvent",
								"code": ";"
							}
						]
					}
				],
				"events": [
					{
						"id": 4,
						"name": "WidgetOnEnter",
						"code": ";"
					}
				],
				"fields": [
					{
						"id": 5,
						"name": "TEST",
						"label": "Default label"
					}
				],
				"styles": [
					{
						"id": 6,
						"condition": "row.id % 2 == 0",
						"cssClass": "fuck"
					}
				],
				"fieldGroups": [
					{
						"id": 7,
						"name": "ORG",
						"label": "Dados da organização",
						"collapsible": true,
						"opened": true
					}
				],
				"widgets": [
					{
						"id": 100,
						"name": "secondWidget",
						"label": "Second widget",
						"events": [
							{
								"id": 102,
								"name": "WidgetOnEnter",
								"code": ";"
							}
						]
					}
				]
			}
		]
	};

	var defaultContainerPreference = {
		"name": "login",
		"showFooter": false,
		"widgets": [
			{
				"id": 1,
				"name": "loginWidget",
				"template": "widget/form.html",
				"actions": [
					{
						"id": 2,
						"name": "TEST",
						"showAsAction": "never",
						"events": [
							{
								"id": 3,
								"name": "ActionEvent",
								"code": "args.owner.widget.newRow();"
							}
						]
					}
				],
				"events": [
					{
						"id": 4,
						"name": "WidgetOnEnter",
						"code": "args.owner.newRow();"
					}
				],
				"fields": [
					{
						"id": 5,
						"name": "TEST",
						"label": "Preference label"
					},{
						"id": 20,
						"name": "NEW_FIELD",
						"label": "New field"
					}
				],
				"styles" : [
					{
						"id": 6,
						"condition" : "row.ID % 2 === 0",
						"cssClass" : "zh-inactive"
					}
				],
				"fieldGroups": [
					{
						"id": 7,
						"name": "ORG changed",
						"label": "Dados da organização",
						"collapsible": true,
						"opened": false
					}
				],
				"widgets": [
					{
						"id": 1000,
						"name": "thirdWidget",
						"label": "3"
					},
					{
						"id": 100,
						"name": "secondWidget",
						"label": "2",
						"fields": [
							{
								"id": 101,
								"name": "second TEST",
								"label": "Second widget field label"
							}
						],
						"events": [
							{
								"id": 102,
								"name": "WidgetOnEnter",
								"code": "args.owner.newRow();"
							}
						]
					}
				]
			}
		]
	};

	var buildMock = function() {
		var mock = {
			whenSuccess: function() {},
			whenError: function() {}
		};
		spyOn(mock, 'whenSuccess').and.callThrough();
		spyOn(mock, 'whenError').and.callThrough();
		return mock;
	};

	beforeEach(function(){
		requestDataSourceEngine = {
			save: function(){
				return thenable();
			}
		};
		localStorageEngine = {
			save: function(){
				return thenable();
			},
			getDataSource: function(){
				return thenable([defaultContainerPreference]);
			},
			sync: function(){
				return thenable([defaultContainerPreference]);
			}
		};

		var repositoryFactory = {
			factory: function(){
				return {
					find: function(){
						return zhPromise.when([defaultContainerPreference]);
					},
					save: function(){
						return zhPromise.when();
					}
				};
			}
		};

		var zhUtil = {
			registerDataSource : function(name){
				repoTest = name;
		}};

		synchronizableStorageEngineFactory = new SynchronizableStorageEngineFactory(zhPromise);
		preferenceManager = new PreferenceManager(synchronizableStorageEngineFactory, {}, localStorageEngine, requestDataSourceEngine, new QueryBuilder(), { i18n: function(string){return string;}}, zhPromise, zhUtil, repositoryFactory);
		containerRepository = new ContainerRepository(preferenceManager);
		containerDecoratorPreference = new ContainerDecoratorPreference(containerRepository, screenService, zhPromise, new ContainerMerger());
	});

	it("Method 'apply': Should return a promise", function(done){
		var fn = function(container){
			expect(container.name).toBe('login');

			done();
		};
		containerDecoratorPreference.apply('login', defaultContainer).then(fn, fn);
	});

	it("Method 'apply': If doesn't pass a valid container name, should rejected a promise", function(done){
		var applyException1 = false;
		var applyException2	= false;
		var promises = [];

		promises.push(containerDecoratorPreference.apply().then(null, function(){ applyException1 = true; }));
		promises.push(containerDecoratorPreference.apply('    ').then(null, function(){ applyException2 = true; }));

		zhPromise.all(promises).then(function(){
			expect(applyException1).toBe(true);
			expect(applyException2).toBe(true);

			done();
		});
	});

	it("Method 'apply': If doesn't pass a valid container object, should rejected a promise", function(done){
		var applyException1 = false;
		var applyException2	= false;
		var promises = [];

		containerDecoratorPreference.apply('login').then(null, function(){ applyException1 = true; });
		containerDecoratorPreference.apply('login', []).then(null, function(){ applyException2 = true; });

		zhPromise.all(promises).then(function(){
			expect(applyException1).toBe(true);
			expect(applyException2).toBe(true);

			done();
		});
	});

	it("Method 'apply': Must merge your concrete container with your preferences", function(done){
		var startTime = new Date().getTime();
		containerDecoratorPreference.apply('login', defaultContainer).then(function(mergedContainer){
			console.log("Time spent: " + ((new Date().getTime()) - startTime) + "ms");
			expect(mergedContainer.showFooter).toEqual(false);
			expect(mergedContainer.showMenu).toEqual(true);
			expect(mergedContainer.widgets[0].template).toEqual('widget/form.html');
			expect(mergedContainer.widgets[0].actions[0].showAsAction).toEqual('never');
			expect(mergedContainer.widgets[0].actions[0].events[0].code).toEqual('args.owner.widget.newRow();');
			expect(mergedContainer.widgets[0].events[0].code).toEqual('args.owner.newRow();');
			expect(mergedContainer.widgets[0].fields.length).toEqual(2);
			expect(mergedContainer.widgets[0].fields[0].label).toEqual('Preference label');
			expect(mergedContainer.widgets[0].fields[1].name).toEqual('NEW_FIELD');
			expect(mergedContainer.widgets[0].fields[1].label).toEqual('New field');
			expect(mergedContainer.widgets[0].styles[0].cssClass).toEqual('zh-inactive');
			expect(mergedContainer.widgets[0].fieldGroups[0].name).toEqual('ORG changed');
			expect(mergedContainer.widgets[0].fieldGroups[0].opened).toEqual(false);
			expect(mergedContainer.widgets[0].widgets.length).toEqual(2);
			expect(mergedContainer.widgets[0].widgets[0].id).toEqual(100);
			expect(mergedContainer.widgets[0].widgets[0].name).toEqual('secondWidget');
			expect(mergedContainer.widgets[0].widgets[0].label).toEqual('2');
			expect(mergedContainer.widgets[0].widgets[0].fields.length).toEqual(1);
			expect(mergedContainer.widgets[0].widgets[0].events.length).toEqual(1);
			expect(mergedContainer.widgets[0].widgets[0].events[0].id).toEqual(102);
			expect(mergedContainer.widgets[0].widgets[0].events[0].code).toEqual('args.owner.newRow();');
			expect(mergedContainer.widgets[0].widgets[1].id).toEqual(1000);
			expect(mergedContainer.widgets[0].widgets[1].name).toEqual('thirdWidget');
			expect(mergedContainer.widgets[0].widgets[1].label).toEqual('3');

			done();
		});
	});

});