describe("NonFunctional IndexedDBEngine", function() {
	if (Util.hasIndexdDBSuport()) {
		var queryBuilder = new QueryBuilder();
		var zhPromise = new ZHPromise(Q);
		var defered = Q.defer();
		defered.resolve({});
		var emptyObject = {};
		var templateManager = {
			getDataSources: function() {
				return defered.promise;
			},
			project: {
				databaseName: "indexedDBTestt"
			},
			container: {
				name: 'container',
				getActiveWidget: function() {
					return {
						name: 'widget'
					};
				}
			}
		};
		var dataToReturn = {
			"registerCountries": {
				"localKey": ["id"],
				"indexes": [{
					name: "indexA",
					"keyPath": "name"
				}]
			}
		};
		var dataSourceHolder = new DataSourceHolder(zhPromise, {
			requestMetaData: function(filename, data, callBack) {
				if (callBack) {
					callBack(dataToReturn);
				}
				var promise = {
					then: function(fn) {
						if (fn) {
							return fn(dataToReturn);
						} else {
							return promise;
						}
					}
				};
				return promise;
			}
		}, console);
		var zhUtil = new ZhUtil({});
		var localStorageEngine = new LocalStorageEngine(console, emptyObject, emptyObject, emptyObject,
			templateManager, {
				dataSources: [],
				clearName: function() {
					return arguments[0];
				},
				getDataSource: function() {
					return {
						then: function(fn) {
							fn({});
						}
					};
				}
			}, zhPromise, zhUtil, queryBuilder);
		var indexedDB = new IndexedDBEngine(dataSourceHolder, emptyObject, emptyObject, emptyObject,
			templateManager, zhPromise, console, zhUtil, queryBuilder);
		var repositoryFactory = new RepositoryFactory(emptyObject, localStorageEngine, emptyObject,
			templateManager, dataSourceHolder, console, {
				bindEvents: function() {}
			}, indexedDB);
		var countriesRepository = repositoryFactory.factory("/registerCountries", "INDEXEDDB");
		beforeEach(function() {
			//countriesRepository.clearAll();
		});

		it("IndexedDBEngine need storage 20 in less than 1 secound using individual save!", function(
			done) {
			
			var processed = false;
			var promises = [];
			countriesRepository.clearAll().then(function() {
				var starTime = new Date().getTime();
				for (var i = 0; i < 20; i++) {
					promises.push(countriesRepository.save({
						"id": i,
						"name": 'Afghanistan ' + i
					}));
				}
				expect(promises.length).toEqual(20);

				zhPromise.all(promises).then(function() {
					var endTime = new Date().getTime();
					var expendTime = (endTime - starTime);
					expect(expendTime).toBeLessThan(1000);
					console.log(
						"IndexedDBEngine need storage 20 in less than 1 secound using individual save!: " +
						expendTime + "ms");
					return countriesRepository.findAll();
				}).then(function(result) {
					expect(result.length).toBe(20);
					processed = true;
					done();
				});
			});
		
		});
		it("IndexedDBEngine need search a iten em 4000 in less than 250 milisecounds", function(done) {
			var processed = false;
			var data = [];
			for (var i = 0; i < 4000; i++) {
				if (i == 2000) {
					data.push({
						"id": i,
						"name": 'Teste Search'
					});
				} else {
					data.push({
						"id": i,
						"name": 'Afghanistan'
					});
				}
			}
			var starTime;
			countriesRepository.clearAll().then(function() {
				return countriesRepository.save(data);
			}).then(function() {
				var where = queryBuilder.build().where("name").equals('Teste Search');
				starTime = new Date().getTime();
				return countriesRepository.findOne(where).then(function(data) {
					expect(data.name).toBe('Teste Search');
				});
			}).then(function() {
				var endTime = new Date().getTime();
				var expendTime = (endTime - starTime);
				console.log("IndexedDBEngine need search a iten em 4000 in less than 250 milisecounds: " +
					expendTime + "ms");
				expect(expendTime).toBeLessThan(250);
				processed = true;
				done();
			});

		});
		it("IndexedDBEngine need storage 6500 in less than 1 secound using bat process", function(done) {
			var processed = false;
			var data = [];
			var starTime;
			for (var i = 0; i < 6500; i++) {
				data.push({
					"id": i,
					"name": 'Afghanistan'
				});
			}
			countriesRepository.clearAll().then(function() {
				starTime = new Date().getTime();
				expect(data.length).toEqual(6500);
				return countriesRepository.save(data);
			}).then(function() {
				var endTime = new Date().getTime();
				var expendTime = (endTime - starTime);
				console.log(
					"IndexedDBEngine need storage 6500 in less than 1 secound using bat process: " +
					expendTime + "ms");
				expect(expendTime).toBeLessThan(1000);
				return countriesRepository.findAll();
			}).then(function(result) {
				expect(result.length).toBe(6500);
				processed = true;
				done();
			});

		});
	}
});